import argparse
import os
import sys

from clint.textui import puts, colored

from photostash import __version__
from photostash.runner import Runner
from photostash.exceptions import CommandError


def main(argv=sys.argv, runner_class=Runner):
    parser = argparse.ArgumentParser(prog='stash')
    parser.add_argument(
        '--version',
        action='version',
        version='%%(prog)s %s' % __version__,
    )

    commands = {}

    command_parsers = parser.add_subparsers(dest='command')

    commands['create'] = command_parsers.add_parser('create')
    commands['create'].add_argument('album_name')

    commands['list'] = command_parsers.add_parser('list')
    commands['list'].add_argument('album_name')

    commands['delete'] = command_parsers.add_parser('delete')
    commands['delete'].add_argument('album_name')
    commands['delete'].add_argument('photo_id')

    commands['add'] = command_parsers.add_parser('add')
    commands['add'].add_argument('album_name')
    commands['add'].add_argument('photo_path')

    commands['open'] = command_parsers.add_parser('open')
    commands['open'].add_argument('photo_id')

    commands['stats'] = command_parsers.add_parser('stats')

    commend = parser.parse_args(argv[1:]).command

    subargs = commands[commend].parse_args(argv[2:])

    runner = runner_class(base_url=os.environ.get('STASH_API_URL', None))

    try:
        getattr(runner, commend)(**dict(subargs._get_kwargs()))
    except CommandError as e:
        puts(colored.red('%s' % e.message), stream=sys.stderr.write)
        sys.exit(1)



from unittest import TestCase

from mock import patch, Mock

from photostash.cli import main, sys
from photostash.exceptions import CommandError


class TestCli(TestCase):

    def call_command(self, args, runner=None):
        if runner is None:
            runner = Mock()
        main(argv=args, runner_class=Mock(return_value=runner))
        return runner

    def test_create_album(self):
        runner = self.call_command(['stash', 'create', 'album'])
        runner.create.assert_called_with(album_name='album')

    def test_list_photo_for_album(self):
        runner = self.call_command(['stash', 'list', 'album'])
        runner.list.assert_called_with(album_name='album')

    def test_delete_photo_from_album(self):
        runner = self.call_command(['stash', 'delete', 'album', 'id'])
        runner.delete.assert_called_with(album_name='album', photo_id='id')

    def test_add_photo_to_album(self):
        runner = self.call_command(['stash', 'add', 'album', 'path'])
        runner.add.assert_called_with(album_name='album', photo_path='path')

    @patch.object(sys, 'exit')
    @patch.object(sys, 'stderr')
    def test_handles_validation_errors(self, mock_stderr, mock_exit):
        runner = Mock()
        runner.create.side_effect = CommandError('Error message')
        self.call_command(['stash', 'create', 'invalid name'], runner=runner)
        mock_exit.called_with(1)
        mock_stderr.write.called_with('Error message')



from unittest import TestCase

from mock import patch, Mock, MagicMock
from slumber.exceptions import HttpServerError
from queryset_client.client import ObjectDoesNotExist

from photostash.exceptions import CommandError
from photostash.runner import Runner


class RunnerTestCase(TestCase):

    def setUp(self):
        self.url = 'http://hostname/'
        self.mock_client = Mock()
        self.mock_client_class = Mock(return_value=self.mock_client)
        self.mock_stdout = Mock()
        self.mock_album = Mock()
        self.mock_album.id = 1
        self.mock_album.name = 'myphotos'
        self.mock_photo = Mock(id=1, image='image')
        self.runner = Runner(self.url, self.mock_client_class, self.mock_stdout)

    def test_create_with_valid_data(self):
        self.mock_client.albums.objects.create.return_value = self.mock_album

        self.runner.create(album_name='myphotos')

        self.mock_client.albums.objects.create.assert_called_with(name='myphotos')
        self.mock_stdout.assert_called_with('myphotos has been created.\n')

    def test_create_with_error_raises_command_error(self):
        self.mock_client.albums.objects.create.side_effect = HttpServerError

        with self.assertRaisesRegexp(CommandError, 'myphotos already exists.'):
            self.runner.create(album_name='myphotos')

    def test_list_photos_for_album(self):
        self.mock_client.albums.objects.get.return_value = self.mock_album
        self.mock_client.photos.objects.filter.return_value = [self.mock_photo]

        self.runner.list(album_name='myphotos')

        self.mock_client.photos.objects.filter.assert_called_with(albumphotos__album=1)
        self.mock_stdout.assert_called_with('Photos: 1\n')

    def test_list_with_no_photos(self):
        self.mock_client.albums.objects.get.return_value = self.mock_album
        self.mock_client.photos.objects.filter.return_value = []

        self.runner.list(album_name='myphotos')

        self.mock_stdout.assert_called_with('type "stash add myphotos <path>\n')

    def test_deletes_album_photo(self):
        mock_album_photo = Mock()
        self.mock_client.albums.objects.get.return_value = self.mock_album
        self.mock_client.photos.objects.get.return_value = self.mock_photo
        self.mock_client.albumphotos.objects.filter.return_value = [mock_album_photo]

        self.runner.delete(album_name='myphotos', photo_id=1)

        mock_album_photo.delete.assert_called()
        self.mock_client.albumphotos.objects.filter.assert_called_with(album=1, photo=1)
        self.mock_stdout.assert_called_with('1 has been removed from myphotos.\n')

    def test_delete_when_album_photo_does_not_exist(self):
        self.mock_client.albums.objects.get.return_value = self.mock_album
        self.mock_client.photos.objects.get.return_value = self.mock_photo
        self.mock_client.albumphotos.objects.filter.return_value = []

        with self.assertRaisesRegexp(CommandError, '1 doest not belong to myphotos'):
            self.runner.delete(album_name='myphotos', photo_id=1)

    @patch('__builtin__.open')
    def test_add_with_photo(self, mock_open):
        mock_open.return_value = MagicMock(spec=file)
        mock_file = mock_open.return_value.__enter__.return_value
        mock_file.read.return_value = 'content'
        self.mock_client.albums.objects.get.return_value = self.mock_album
        self.mock_client.photos.objects.create.return_value = self.mock_photo

        self.runner.add(album_name='myphotos', photo_path='path')

        self.mock_client.photos.objects.create.assert_called_with(image='path:Y29udGVudA==')
        self.mock_client.albumphotos.objects.create.assert_called_with(
            album=self.mock_album,
            photo=self.mock_photo,
        )
        self.mock_stdout.assert_called_with('1 has been added to myphotos.\n')

    @patch('__builtin__.open')
    def test_add_with_photo_path_does_not_exist(self, mock_open):
        mock_open.side_effect = IOError()
        self.mock_client.photos.objects.get.side_effect = CommandError

        with self.assertRaises(CommandError):
            self.runner.add(album_name='myphotos', photo_path='path')

    def test_add_with_photo_id(self):
        self.mock_client.photos.objects.get.return_value = self.mock_photo

        self.runner.add(album_name='myphotos', photo_path='1')

        self.mock_client.photos.objects.get.assert_called_with(id='1')

    @patch('photostash.runner.subprocess')
    def test_open_calls_subprocess(self, mock_subprocess):
        self.mock_client.photos.objects.get.return_value = self.mock_photo

        self.runner.open(photo_id=1)

        mock_subprocess.call.assert_called_with(['open', self.mock_photo.image])

    def test_get_album_returns_album(self):
        self.mock_client.albums.objects.get.side_effect = ObjectDoesNotExist

        regex = r'Album myphotos does not exist. type "stash create myphotos" to add this album.'
        with self.assertRaisesRegexp(CommandError, regex):
            self.runner._get_album(album_name='myphotos')

    def test_get_photo_returns_album(self):
        self.mock_client.photos.objects.get.side_effect = ObjectDoesNotExist

        regex = r'Photo #1 does not exist.'
        with self.assertRaisesRegexp(CommandError, regex):
            self.runner.delete(album_name='myphotos', photo_id=1)



import sys

from setuptools import setup, find_packages

from photostash import __version__


install_requires = [
    'clint',
    'tastypie-queryset-client',
]

if sys.version_info < (2, 7):
    install_requires.append('argparse==1.1')


setup(
    name='photostash-client',
    version=__version__,
    description='Official photostash command line client',
    url='https://github.com/seanbrant/photostash-client',
    author='Sean Brant',
    author_email='brant.sean@gmail.com',
    packages=find_packages(),
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'stash = photostash.cli:main',
        ],
    },
    install_requires=install_requires,
)



import base64
import sys
import subprocess

from clint.textui import puts, columns
from queryset_client import Client
from queryset_client.client import ObjectDoesNotExist
from slumber.exceptions import HttpServerError

from photostash.exceptions import CommandError


class Runner(object):
    DEFULT_BASE_URL = 'http://photostash.herokuapp.com/api/v1/'

    def __init__(self, base_url=None, client=Client, stream=sys.stdout.write):
        if base_url is None:
            base_url = self.DEFULT_BASE_URL
        self.base_url = base_url
        self.client = client(self.base_url)
        self.stream = stream

    def create(self, album_name):
        try:
            album = self.client.albums.objects.create(name=album_name)
            puts('%s has been created.' % album.name, stream=self.stream)
        except HttpServerError:
            raise CommandError('%s already exists.' % album_name)

    def list(self, album_name):
        album = self._get_album(album_name)
        photos = self._get_photos(album)

        ids = ', '.join([str(photo.id) for photo in photos])

        if ids:
            puts('Photos: %s' % ids, stream=self.stream)
        else:
            puts('%s has no photos.' % album.name, stream=self.stream)
            puts('type "stash add %s <path>' % album.name, stream=self.stream)

    def delete(self, album_name, photo_id):
        album = self._get_album(album_name)
        photo = self._get_photo(photo_id)

        try:
            rel = self.client.albumphotos.objects.filter(photo=photo.id, album=album.id)[0]
            rel.delete()
            puts('%s has been removed from %s.' % (photo.id, album.name), stream=self.stream)
        except IndexError:
            raise CommandError('%s doest not belong to %s.' % (photo.id, album.name))

    def add(self, album_name, photo_path):
        album = self._get_album(album_name)

        try:
            with open(photo_path, 'rb') as fp:
                image = '%s:%s' % (photo_path, base64.b64encode(fp.read()))
                photo = self.client.photos.objects.create(image=image)
        except IOError as e:
            try:
                photo = self._get_photo(photo_path)
            except CommandError:
                raise CommandError(e)

        self.client.albumphotos.objects.create(album=album, photo=photo)

        puts('%s has been added to %s.' % (photo.id, album.name), stream=self.stream)

    def open(self, photo_id):
        photo = self._get_photo(photo_id)
        puts('Opening %s...' % photo.id, stream=self.stream)
        subprocess.call(['open', photo.image])

    def stats(self):
        albums = self.client.albums.objects.all()
        stats = sorted((album.name, self._get_photos(album)) for album in albums)

        names = ['| Album'] + ['| {0}'.format(album) for album, photos in stats]
        col1 = ['\n'.join(names), max([len(name) for name in names]) + 1]

        photos = ['| Photos'] + ['| {0}'.format(', '.join([photo.id for photo in photos]))
                  for album, photos in stats]
        col2 = ['\n'.join(photos), max([len(photo) for photo in photos]) + 1]

        col3 = ['\n'.join(['|' for i in range(len(stats) + 1)]), None]
        table = columns(col1, col2, col3)

        rows = table.splitlines()

        header = rows.pop(0)
        divider = '-' * len(header.strip())

        puts(divider, stream=self.stream)
        puts(header, stream=self.stream)
        puts(divider, stream=self.stream)
        puts('\n'.join(rows), stream=self.stream)
        puts(divider, stream=self.stream)

    def _get_album(self, album_name):
        try:
            return self.client.albums.objects.get(name=album_name)
        except ObjectDoesNotExist:
            raise CommandError('Album {0} does not exist. type "stash create '
                               '{0}" to add this album.'.format(album_name))

    def _get_photo(self, photo_id):
        try:
            return self.client.photos.objects.get(id=photo_id)
        except ObjectDoesNotExist:
            raise CommandError('Photo #{0} does not exist.'.format(photo_id))

    def _get_photos(self, album):
        return self.client.photos.objects.filter(albumphotos__album=album.id)
